@@ -8,7 +8,8 @@ class JsonProperty(object):
     default = None
 
     def __init__(self, default=Ellipsis, name=None, choices=None, required=False,
-                 exclude_if_none=False, validators=()):
+                 exclude_if_none=False, validators=None, verbose_name=None):
+        validators = validators or ()
         self.name = name
         if default is Ellipsis:
             default = self.default
@@ -17,6 +18,12 @@ class JsonProperty(object):
         else:
             self.default = lambda: default
         self.choices = choices
+        self.choice_keys = []
+        if choices:
+            for choice in choices:
+                if isinstance(choice, tuple):
+                    choice, _ = choice
+                self.choice_keys.append(choice)
         self.required = required
         self.exclude_if_none = exclude_if_none
         if hasattr(validators, '__iter__'):
@@ -26,6 +33,7 @@ class JsonProperty(object):
             self.custom_validator = _validator
         else:
             self.custom_validator = validators
+        self.verbose_name = verbose_name
 
     def init_property(self, default_name):
         self.name = self.name or default_name
@@ -53,8 +61,11 @@ class JsonProperty(object):
         return self.wrap(value)
 
     def __get__(self, instance, owner):
-        assert self.name in instance
-        return instance[self.name]
+        if instance:
+            assert self.name in instance
+            return instance[self.name]
+        else:
+            return self
 
     def __set__(self, instance, value):
         instance[self.name] = value
@@ -80,12 +91,14 @@ class JsonProperty(object):
         return value is None
 
     def validate(self, value, required=True):
-        if self.choices and value not in self.choices and value is not None:
+        if self.choice_keys and value not in self.choice_keys and value is not None:
             raise BadValueError(
-                '{0!r} not in choices: {1!r}'.format(value, self.choices)
+                '{0!r} not in choices: {1!r}'.format(value, self.choice_keys)
             )
-        self.custom_validator(value)
-        if required and self.empty(value) and self.required:
+
+        if not self.empty(value):
+            self.custom_validator(value)
+        elif required and self.required:
             raise BadValueError(
                 'Property {0} is required.'.format(self.name)
             )
@@ -102,8 +115,8 @@ class JsonContainerProperty(JsonProperty):
         from convert import ALLOWED_PROPERTY_TYPES
         if inspect.isfunction(item_type):
             item_type = item_type()
-        else:
-            item_type = item_type
+        if hasattr(item_type, '_type'):
+            item_type = item_type._type
         self.item_type = item_type
         if item_type and item_type not in tuple(ALLOWED_PROPERTY_TYPES) \
                 and not issubclass(item_type, JsonObject):
@@ -136,6 +149,7 @@ class JsonContainerProperty(JsonProperty):
     def _update(self, container, extension):
         raise NotImplementedError()
 
+
 class DefaultProperty(JsonProperty):
     def wrap(self, obj):
         from . import convert
@@ -242,6 +256,10 @@ class JsonArray(list):
         self._obj.append(unwrapped)
         super(JsonArray, self).append(wrapped)
 
+    def __delitem__(self, i):
+        super(JsonArray, self).__delitem__(i)
+        del self._obj[i]
+
     def extend(self, wrapped_list):
         if wrapped_list:
             wrapped_list, unwrapped_list = zip(
@@ -508,7 +526,7 @@ class _JsonObjectPrivateInstanceVariables(object):
         self.dynamic_properties = dynamic_properties or {}
 
 
-class JsonObject(SimpleDict):
+class JsonObjectBase(object):
 
     __metaclass__ = JsonObjectMeta
 
@@ -519,15 +537,14 @@ class JsonObject(SimpleDict):
     _properties_by_key = None
 
     def __init__(self, _obj=None, **kwargs):
-        super(JsonObject, self).__init__()
-
         setattr(self, '_$', _JsonObjectPrivateInstanceVariables())
 
         self._obj = check_type(_obj, dict,
                                'JsonObject must wrap a dict or None')
+        self._wrapped = {}
 
         for key, value in self._obj.items():
-           self.set_raw_value(key, value)
+            self.set_raw_value(key, value)
 
         for attr, value in kwargs.items():
             setattr(self, attr, value)
@@ -547,6 +564,9 @@ class JsonObject(SimpleDict):
         else:
             setattr(self, key, wrapped)
 
+    def properties(self):
+        return self._properties_by_attr.copy()
+
     @property
     def __dynamic_properties(self):
         return getattr(self, '_$').dynamic_properties
@@ -557,7 +577,7 @@ class JsonObject(SimpleDict):
         return self
 
     def validate(self, required=True):
-        for key, value in self.items():
+        for key, value in self._wrapped.items():
             self.__get_property(key).validate(value, required=required)
 
     def to_json(self):
@@ -589,7 +609,7 @@ class JsonObject(SimpleDict):
 
     def __setitem__(self, key, value):
         wrapped, unwrapped = self.__unwrap(key, value)
-        super(JsonObject, self).__setitem__(key, wrapped)
+        self._wrapped[key] = wrapped
         if self.__get_property(key).exclude(unwrapped):
             self._obj.pop(key, None)
         else:
@@ -597,12 +617,13 @@ class JsonObject(SimpleDict):
         if key not in self._properties_by_key:
             assert key not in self._properties_by_attr
             self.__dynamic_properties[key] = wrapped
-            super(JsonObject, self).__setattr__(key, wrapped)
+            super(JsonObjectBase, self).__setattr__(key, wrapped)
 
     def __is_dynamic_property(self, name):
         return (
             name not in self._properties_by_attr and
-            not name.startswith('_')
+            not name.startswith('_') and
+            not hasattr(self.__class__, name)
         )
 
     def __setattr__(self, name, value):
@@ -615,27 +636,32 @@ class JsonObject(SimpleDict):
                     "(not a valid property)".format(name)
                 )
         else:
-            super(JsonObject, self).__setattr__(name, value)
+            super(JsonObjectBase, self).__setattr__(name, value)
 
     def __delitem__(self, key):
         if key in self._properties_by_key:
             raise DeleteNotAllowed(key)
         else:
+            if not self.__is_dynamic_property(key):
+                raise KeyError(key)
             del self._obj[key]
-            super(JsonObject, self).__delitem__(key)
-            super(JsonObject, self).__delattr__(key)
+            del self._wrapped[key]
+            del self.__dynamic_properties[key]
+            super(JsonObjectBase, self).__delattr__(key)
 
     def __delattr__(self, name):
         if name in self._properties_by_attr:
             raise DeleteNotAllowed(name)
-        else:
+        elif self.__is_dynamic_property(name):
             del self[name]
+        else:
+            super(JsonObjectBase, self).__delattr__(name)
 
     def __repr__(self):
         name = self.__class__.__name__
         predefined_properties = self._properties_by_attr.keys()
         predefined_property_keys = set(self._properties_by_attr[p].name for p in predefined_properties)
-        dynamic_properties = set(self.keys()) - predefined_property_keys
+        dynamic_properties = set(self._wrapped.keys()) - predefined_property_keys
         properties = sorted(predefined_properties) + sorted(dynamic_properties)
         return u'{name}({keyword_args})'.format(
             name=name,
@@ -646,5 +672,42 @@ class JsonObject(SimpleDict):
         )
 
 
+class _LimitedDictInterfaceMixin(object):
+    """
+    mindlessly farms selected dict methods out to an internal dict
+
+    really only a separate class from JsonObject
+    to keep this mindlessness separate from the methods
+    that need to be more carefully understood
+
+    """
+    _wrapped = None
+
+    def keys(self):
+        return self._wrapped.keys()
+
+    def items(self):
+        return self._wrapped.items()
+
+    def iteritems(self):
+        return self._wrapped.iteritems()
+
+    def __contains__(self, item):
+        return item in self._wrapped
+
+    def __getitem__(self, item):
+        return self._wrapped[item]
+
+    def __iter__(self):
+        return iter(self._wrapped)
+
+    def __len__(self):
+        return len(self._wrapped)
+
+
+class JsonObject(JsonObjectBase, _LimitedDictInterfaceMixin):
+    pass
+
+
 def get_dynamic_properties(obj):
     return getattr(obj, '_$').dynamic_properties.copy()
