@@ -11,7 +11,7 @@ class Features(JsonObject):
 
     """
 
-    hair = StringProperty(choices=['brown', 'blond', 'grey'])
+    hair = StringProperty(choices=['brown', ('blond', 'Blond'), 'grey'])
     eyes = StringProperty()
 
 
@@ -119,7 +119,7 @@ class JsonObjectTestCase(unittest2.TestCase):
 
         features = Features.wrap(features)
         danny.features = features
-        self.assertEqual(danny.features, {'hair': 'grey', 'eyes': 'blue'})
+        self.assertEqual(dict(danny.features), {'hair': 'grey', 'eyes': 'blue'})
 
         numbers = [1, 2, 3, 4, 5]
         danny.favorite_numbers = numbers
@@ -299,6 +299,13 @@ class JsonObjectTestCase(unittest2.TestCase):
             }
         })
 
+    def test_access_to_descriptor(self):
+        p = StringProperty()
+        class Foo(JsonObject):
+            string = p
+
+        self.assertIs(Foo.string, p)
+
 
 class LazyValidationTest(unittest2.TestCase):
 
@@ -417,6 +424,18 @@ class PropertyTestCase(unittest2.TestCase):
         self.assertEqual(foo.decimal, decimal.Decimal('2.0'))
         self.assertEqual(foo.to_json()['decimal'], '2.0')
 
+        foo.decimal = 3
+        self.assertEqual(foo.decimal, decimal.Decimal(3L))
+        self.assertEqual(foo.to_json()['decimal'], '3')
+
+        foo.decimal = 4L
+        self.assertEqual(foo.decimal, decimal.Decimal(4L))
+        self.assertEqual(foo.to_json()['decimal'], '4')
+
+        foo.decimal = 5.25
+        self.assertEqual(foo.decimal, decimal.Decimal(unicode(5.25)))
+        self.assertEqual(foo.to_json()['decimal'], '5.25')
+
     def test_dict(self):
         mapping = {'one': 1, 'two': 2}
         o = ObjectWithDictProperty(mapping=mapping)
@@ -452,7 +471,7 @@ class PropertyTestCase(unittest2.TestCase):
         with self.assertRaises(AttributeError):
             foo.blah = 3
         foo._blah = 5
-        self.assertEqual(foo, {})
+        self.assertEqual(dict(foo), {})
         self.assertEqual(foo.to_json(), {})
         self.assertEqual(foo._blah, 5)
 
@@ -465,11 +484,49 @@ class PropertyTestCase(unittest2.TestCase):
         self.assertEqual(foo.to_json(), {'name': None})
         self.assertEqual(foo._id, None)
         foo = Foo(_id='xxx')
-        self.assertEqual(foo, {'name': None, '_id': 'xxx'})
+        self.assertEqual(dict(foo), {'name': None, '_id': 'xxx'})
         foo._id = None
         self.assertEqual(foo.to_json(), {'name': None})
         self.assertEqual(foo._id, None)
 
+    def test_descriptor(self):
+        class Desc(object):
+            def __get__(self, instance, owner):
+                if not instance:
+                    return self
+                return instance._string
+
+            def __set__(self, instance, value):
+                instance._string = value
+
+        class Foo(JsonObject):
+            _string = StringProperty()
+            string = Desc()
+
+        foo = Foo(_string='hello')
+        self.assertEqual(foo._string, 'hello')
+        self.assertEqual(foo.string, 'hello')
+        foo.string = 'goodbye'
+        self.assertEqual(foo.string, 'goodbye')
+        self.assertEqual(foo._string, 'goodbye')
+        self.assertEqual(foo.to_json(), {
+            '_string': 'goodbye',
+        })
+    def test_key_error(self):
+        class Foo(JsonObject):
+            pass
+        foo = Foo()
+
+        with self.assertRaises(KeyError):
+            foo['hello']
+
+    def test_attribute_error(self):
+        class Foo(JsonObject):
+            pass
+        foo = Foo()
+        
+        with self.assertRaises(AttributeError):
+            foo.hello
 
 class DynamicConversionTestCase(unittest2.TestCase):
     import datetime
@@ -482,7 +539,6 @@ class DynamicConversionTestCase(unittest2.TestCase):
     def _test_dynamic_conversion(self, foo):
         string_date = self.string_date
         date_date = self.date_date
-
         self.assertEqual(foo.to_json()['my_date'], string_date)
         self.assertEqual(foo.my_date, date_date)
 
